/**
 * @fileoverview 自定义在项目中禁止字符串/字符
 * @author XuFengLin
 */
"use strict";
/** @type {import('eslint').Rule.RuleModule} */
module.exports = {
  meta: {
    docs: {
      description: "项目中禁止出现指定的字符组合",
    },
    messages: {
      //提示信息
      unexpectedLiteral: "字符串中不允许包含禁止字符：{{forbiddenString}}",
    },
    schema: [
      //用户配置规范，配置必须是Array<string>
      {
        type: "array",
        items: {
          type: "string",
        },
        uniqueItems: true,
        minItems: 1,
      },
    ],
    fixable: null, // 出现后修复函数，目前仅提示爆红处理，不进行修复
  },
  create: function (context) {
    // 从规则配置中获取用户指定的字符组合数组
    let forbiddenStrings = context.options[0];
    // forbiddenStrings = ["mola","m"]
    //对forbiddenStrings进行兜底处理
    if (
      !Array.isArray(forbiddenStrings) ||
      forbiddenStrings.length === 0 ||
      !forbiddenStrings.every((str) => typeof str === "string")
    ) {
      //如果依然出现违规配置，则直接返回空配置，防止浪费性能
      return {};
    }

    // 检查字符串是否包含禁止的字符组合
    function checkForForbiddenStrings(node, value) {
      for (const forbiddenString of forbiddenStrings) {
        if (value.includes(forbiddenString)) {
          context.report({
            //提示报错
            node,
            // messageId: "unexpectedLiteral",
            message: "出现违规字符/字符组",
            data: {
              forbiddenString,
            },
          });
          // 匹配到违规字符后提前退出循环
          break;
        }
      }
    }

    return {
      // 匹配所有的 Literal(字面量) 节点
      Literal(node) {
        if (typeof node.value === "string") {
          checkForForbiddenStrings(node, node.value);
        }
      },
      // 匹配所有属性值为 Literal 类型的 Property 节点
      // "Property > Literal.value"(node) {
      //   if (typeof node.value === "string") {
      //     checkForForbiddenStrings(node, node.value);
      //   }
      // },
    };
  },
};
